/*
 * Copyright 2009-2012 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.reqcoverage.eclipse.wizards;

import java.io.ByteArrayInputStream;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.DialogSettings;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.wizards.WorkspaceFileSelectionPage;
import com.unitesk.requality.eclipse.wizards.ITestGenWizard;
import com.unitesk.requality.eclipse.wizards.TreeNodeSelectionPage;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.tests.ProjectDialogSettingsHandler;

public class ExistingTestWizard extends Wizard implements ITestGenWizard {
	private DialogSettings settings = new DialogSettings("newtswizard");
    private FileContentPage page;
    private WorkspaceFileSelectionPage fpage;
    private TreeNodeSelectionPage selnodepage;
    private ProjectDialogSettingsHandler settingsLoader = new ProjectDialogSettingsHandler("existtestwizard");
    private ISelection selection;
    protected String targetqid;
	private TreeDB db;
	@Override
	public IDialogSettings getDialogSettings() {
		return settings;
	};
    /**
     * Constructor for ReqWizard.
     */
    public ExistingTestWizard() {
        super();
        setNeedsProgressMonitor(true);
    }

    /**
     * Adding the page to the wizard.
     */
    public void addPages() {
    	TreeDB db = getSelectedTreeDB();
        StructuredSelection sel;
        if(db != null)
        	sel = new StructuredSelection(db);
        else
        	sel = new StructuredSelection();
        selnodepage = new TreeNodeSelectionPage(sel,"/",SWT.SINGLE,targetqid);
        selnodepage.setNodeTypesFilters(new String[]{TestPurpose.TYPE_NAME, Requirement.TYPE_NAME});
        selnodepage.setChangeListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				if(event == null || event.getSelection().isEmpty())
					setTargetQID(null, false);
				else{
					TreeNode node = ((TreeNode)((StructuredSelection)event.getSelection()).getFirstElement());
					setTargetQID(node.getTreeDB().getFullName(node), false);
				}
			}
		});
        addPage(selnodepage);
        if(targetqid!=null)
        	selnodepage.setTargetQIDs(targetqid, true);
        fpage = new WorkspaceFileSelectionPage(settings,new String[]{"ttcn"});
        addPage(fpage);
        page = new FileContentPage(settings);
        addPage(page);

        if(targetqid!=null)
        	page.setTargetQID(targetqid, true);
        fpage.setChangeListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				page.updateState();
			}
		});
    }

    private boolean writeFileContent(String path,String content){
    	if(path == null || content == null)
    		return false;
    	try {
        	IFile f = ResourcesPlugin.getWorkspace().getRoot().getFile(Path.fromOSString(path));
        	if(f == null || !f.exists())
        		return false;
        	f.setContents(new ByteArrayInputStream(content.getBytes()),0, null);
			return true;
		} catch (CoreException e) {
			e.printStackTrace();
		}
    	return false;
    }
	private TreeDB getSelectedTreeDB() {
    	if(!(selection instanceof StructuredSelection))
    		return db;
    	TreeDB db = this.db;
    	Object o = ((StructuredSelection)selection).getFirstElement();
    	if(o instanceof IProject)
    		db = TreesTracker.getTracker().getTree((IProject)o);
    	if(o instanceof TreeNode)
    		db = ((TreeNode)o).getTreeDB();
    	if(o instanceof TreeDB)
    		db = (TreeDB)o;
    	this.db = db;
    	return db;
	}

	@Override
    public boolean performFinish() {
        try {
        	TreeNode node = db.getNode(targetqid);
        	settingsLoader.saveSettings(settings);
        	writeFileContent(settings.get("filePath"), page.getNewFileContents());
        } catch (Exception e) {
        	db.rollback();
            e.printStackTrace();
        }
        return true;
    }

    /**
     * We will accept the selection in the workbench to see if we can initialize
     * from it.
     * 
     * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
     */
    public void init(IWorkbench workbench, IStructuredSelection selection) {
        this.selection = selection;
    }
	
	@Override
	public String getTargetQID() {
		return targetqid;
	}

	public void setTargetQID(String targetqid, boolean b) {
		if(page!=null && !b)
			page.setTargetQID(targetqid, b);
		if(selnodepage!=null && !b)
			selnodepage.setTargetQIDs(targetqid,true);
		
		this.targetqid = targetqid;		
	}
	
	@Override
	public boolean canFinish() {
		String path = settings.get("filePath"); 
		if(path == null || path.equals("") || ResourcesPlugin.getWorkspace().getRoot().getFile(Path.fromOSString(path))==null)
			return false;
		TreeNode node;
		try{
		//	node = db.getNode(qid);
  			TreeNode node2 = db.getNode(targetqid);
			return node2!=null;
		}catch(Exception e){}
		return false;
	};

	@Override
	public void setTreeDB(TreeDB treeDB) {
		this.db = treeDB;
		settingsLoader.initOverProject(TreesTracker.getTracker().getProject(treeDB));
		settingsLoader.loadSettings(settings);
	}

	@Override
	public Map<String, String> getSettings() {
		// TODO Auto-generated method stub
		return null;
	}
}