package com.unitesk.reqcoverage.tests;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;

import com.unitesk.reqcoverage.Activator;
import com.unitesk.reqcoverage.eclipse.GetPathToTestProjects;
import com.unitesk.reqcoverage.reqstracer.Collector;
import com.unitesk.requality.core.CoreUtils;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tests.CoverageStorage;
import com.unitesk.requality.tests.ICoverageSourceWizard;
import com.unitesk.requality.tests.ICoverageStorageSource;
import com.unitesk.requality.tests.TestErrorInfo;
import com.unitesk.requality.tests.TreeNodeCoverageInfo;

public class CoverageByProjects implements ICoverageStorageSource {
	public static final String ID = "com.unitesk.reqcoverage.fromProject";
	public List<String> extension;
	public List<String> fileregexp;  
	public Collector collector;
	private String lastErrors;
	private List<File> projects;
	private String[] projectsNames;
	private CoverageStorage targetCoverageStorage;

	public String getCurrentContainer(){
		return getCurrentContainer(null);
	}
	public String getCurrentContainer(TreeNode node){
		if(node == null)
			return targetCoverageStorage!=null?TreesTracker.getTracker().getProject(targetCoverageStorage.getTreeDB()).getLocation().toOSString():null;
		IProject p = TreesTracker.getTracker().getProject(node.getTreeDB());
		if(p!=null)
			return p.getLocation().toOSString();
		IResource r = TreesTracker.getResourceByNode(node);
		if(r!=null)
			return r.getProject().getLocation().toOSString();
		return (new java.io.File(node.getTreeDB().getRootNode().getResourcesPath())).getParentFile().getParentFile().getAbsolutePath();
	}
	
	public void setCollector(Collector instance,TreeNode node ){
		collector = instance;
		projects = instance.getTestProjectsList();
		projectsNames = new String[projects.size()];
		int i = 0;
		for(File p :projects)
			projectsNames[i++] = Activator.getRelativePath(getCurrentContainer(node), p);
		fileregexp = instance.getRegExpressions();
		extension = instance.getExtensions();
	}
	public CoverageByProjects(){
		
	}
	
	public CoverageByProjects(Collector instance,TreeNode node) {
		setCollector(instance,node);
	}
	private Collector getCollector(){
		if(collector == null){
			collector = Collector.instance();
			collector.setProcessReport(true);
			collector.setContainersList(new ArrayList<File>(projects));
			collector.setExtensions(new ArrayList<String>(extension));
			collector.setRegExpressions(new ArrayList<String>(fileregexp));			
		}
		return collector;
	}
	@Override
	public Map<String, TreeNodeCoverageInfo> getCoverage() {
		lastErrors = "";
		try {
			if(collector != null)
				collector.reset(); 
			collector = null;
			String ret = getCollector().selectFilesToFindReqs();
			if(!ret.equals(""))
				lastErrors += " some of selected projects causes errors: "+ret.substring(0,ret.length()-2);
			getCollector().constructMapReqsToFiles();
		} catch (CoreException e) {
			lastErrors += " "+e.getMessage();
			e.printStackTrace();
		} catch (IOException e) {
			lastErrors += " "+e.getMessage();
			e.printStackTrace();
		}
		Map<String, TreeNodeCoverageInfo> res = new HashMap<String, TreeNodeCoverageInfo>();
		Map<String, ArrayList<String>> map = getCollector().getReqsToFilesMap();
		TreeDB db = targetCoverageStorage.getTreeDB();
		for(String s:map.keySet()){
			String bases = s;
			if(db!=null)
			{
				try{
					s = db.getNode(s).getQualifiedId();
				}
				catch(Exception e){
				}
			}
			TreeNodeCoverageInfo tnci;
			if(!res.containsKey(s))
				tnci = new TreeNodeCoverageInfo();
			else
				tnci = res.get(s);
			tnci.setTarget(s);
			for(String s2:map.get(bases))
				tnci.registerTest("file:"+IPath.SEPARATOR+IPath.SEPARATOR+IPath.SEPARATOR+s2);
			res.put(s, tnci);
		}
		return res;
	}
	@Override
	public void setSettings(String... setts) {
		if(collector != null)
			collector.reset();
		collector = null;
		fileregexp = new ArrayList<String>();
		projects = new ArrayList<File>();
		extension = new ArrayList<String>();
		if(setts.length>=3){
			fileregexp =  Arrays.asList(setts[0].replace("'", "").split("\\|"));
			projectsNames = setts[1].split("\\|");
			if(targetCoverageStorage!=null)
				projects = Activator.getContainersByNames(getCurrentContainer(targetCoverageStorage.getTreeDB().getNode(Requirement.getTypeRootQId())),projectsNames);
			else
				projects = null;
			extension =  Arrays.asList(setts[2].split("\\|"));
		}
		getCollector();
	}
	@Override
	public String getLastErrors() {
		return lastErrors;
	}
	@Override
	public String[] getSettings(TreeNode node) {
		String[] arr = new String[3];
		arr[0] = CoreUtils.listToString(fileregexp, "|");
		arr[1] = "";
		for(File p:projects)
			arr[1]+=Activator.getRelativePath(getCurrentContainer(node), p)+"|";
		if(!arr[1].equals(""))
			arr[1] = arr[1].substring(0,arr[1].length()-1);
		arr[2] = CoreUtils.listToString(extension, "|");
		return arr;
	}
	@Override
	public Map<String, TestErrorInfo[]> getErrors() {
		return new HashMap<String, TestErrorInfo[]>();
	}
	@Override
	public void setTargetStorage(CoverageStorage coverageStorage) {
		targetCoverageStorage = coverageStorage;
		TreeNode node = null;
		try{
			node = targetCoverageStorage.getTreeDB().getNode(Requirement.getTypeRootQId());
		}catch(Exception e){
		}
		if(projectsNames!=null && node!=null)
			projects = Activator.getContainersByNames(getCurrentContainer(node),projectsNames);
	}
	@Override
	public ICoverageSourceWizard getWizard(TreeNode db) {
		ICoverageSourceWizard wiz = new GetPathToTestProjects(this);
		wiz.setCoverageStorageSettings(db,new String[]{"'(/Requirements.*)|@requirement (.*)'","","ttcn"});
		return wiz;
	}
	@Override
	public Object getSourceObject(TreeDB db, Map<String, Object> node) {
		return projects.toArray(new File[0]);
	}

}
